@@ -1,26 +1,18 @@
 try:
     import unittest2 as unittest
-except ImportError:
+except ImportError:  # pragma: no cover
     import unittest
 
-# from functools import partial
 from functools import wraps
 
 
-def get_decorated_method_name(method_name, param_names, param_values):
-    name_components = []
-
-    for k, v in zip(param_names, param_values):
-        name_components.append(str(k))
-        name_components.append(str(v))
-
-    return '%s_%s' % (method_name, '_'.join(name_components))
-
-
-def augment_method_docstring(method, new_class_dict, classname, param_names, param_values, new_method):
-    param_assignments_str = '; '.join(['%s = %s' % (k, v) for (k, v) in zip(param_names, param_values)])
-    extra_doc = "%s (%s.%s) [with %s] " \
-        % (method.__name__, new_class_dict.get('__module__', '<module>'), classname, param_assignments_str)
+def augment_method_docstring(method, new_class_dict, classname,
+                             param_names, param_values, new_method):
+    param_assignments_str = '; '.join(
+        ['%s = %s' % (k, v) for (k, v) in zip(param_names, param_values)])
+    extra_doc = "%s (%s.%s) [with %s] " % (
+        method.__name__, new_class_dict.get('__module__', '<module>'),
+        classname, param_assignments_str)
 
     try:
         new_method.__doc__ = extra_doc + new_method.__doc__
@@ -29,32 +21,49 @@ def augment_method_docstring(method, new_class_dict, classname, param_names, par
 
 
 class ParameterizedTestCaseMetaClass(type):
+    method_counter = {}
 
     def __new__(meta, classname, bases, class_dict):
         new_class_dict = {}
 
         for attr_name, attr_value in class_dict.items():
             if callable(attr_value) and hasattr(attr_value, 'param_names'):
-                # print("Processing attr_name = %r; attr_value = %r" % (attr_name, attr_value))
+                # print("Processing attr_name = %r; attr_value = %r" % (
+                #     attr_name, attr_value))
 
                 method = attr_value
                 param_names = attr_value.param_names
                 data = attr_value.data
+                func_name_format = attr_value.func_name_format
 
-                meta.process_method(classname, method, param_names, data, new_class_dict)
+                meta.process_method(
+                    classname, method, param_names, data, new_class_dict,
+                    func_name_format)
             else:
                 new_class_dict[attr_name] = attr_value
 
         return type.__new__(meta, classname, bases, new_class_dict)
 
     @classmethod
-    def process_method(cls, classname, method, param_names, data, new_class_dict):
+    def process_method(
+            cls, classname, method, param_names, data, new_class_dict,
+            func_name_format):
+        method_counter = cls.method_counter
+
         for param_values in data:
             new_method = cls.new_method(method, param_values)
-            new_method.__name__ = get_decorated_method_name(method.__name__, param_names, param_values)
-            augment_method_docstring(method, new_class_dict, classname, param_names, param_values, new_method)
-            new_class_dict[new_method.__name__] = new_method
+            method_counter[method.__name__] = \
+                method_counter.get(method.__name__, 0) + 1
+            case_data = dict(zip(param_names, param_values))
+            case_data['func_name'] = method.__name__
+            case_data['case_num'] = method_counter[method.__name__]
 
+            new_method.__name__ = func_name_format.format(**case_data)
+
+            augment_method_docstring(
+                method, new_class_dict, classname,
+                param_names, param_values, new_method)
+            new_class_dict[new_method.__name__] = new_method
 
     @classmethod
     def new_method(cls, method, param_values):
@@ -69,7 +78,8 @@ class ParameterizedTestMixin(object):
     __metaclass__ = ParameterizedTestCaseMetaClass
 
     @classmethod
-    def parameterize(cls, param_names, data):
+    def parameterize(cls, param_names, data,
+                     func_name_format='{func_name}_{case_num:05d}'):
         """Decorator for parameterizing a test method - example:
 
         @ParameterizedTestCase.parameterize(
@@ -86,6 +96,7 @@ class ParameterizedTestMixin(object):
 
             newfunc.param_names = param_names
             newfunc.data = data
+            newfunc.func_name_format = func_name_format
 
             return newfunc
 
